/*************************************************************

PROJECT   :  ULTRASONIC HAPTIC VISION

MEMBERS   :	 ROHAN SHARMA, JEFF BUENTE
CODE TYPE :  RECEIVER - RF-RX, RS232 CONTROL
CLASS	  :	 ECE 4760

**************************************************************/


//************************************************************
// Headers Files and other includes
//************************************************************

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include ".\txrx.c"
#include ".\uart.c"

//************************************************************
// Defines
//************************************************************

// Led Port defines 
#define DDRL	DDRC
#define PORTL	PORTC
#define PINL	PINC

// RF Port defines
#define DDRR	DDRD
#define PORTR	PORTD
#define PINR	PIND
#define cpu_l	0x04				// Pin number of CPU led present on the MCU board

// RF Receiver usage defines
#define baudrate		4800		// Baudrate value for UART0 communication with RS-232 and RF Receiver
#define f_cpu 			16000000UL	// Frequency at which the receiver MCU is running at 
#define max_rx_length	20			// Maximum length of the RF packet to be received from the transmitter

#define rx_id		 	0x0A		// ID of packet to be received from the transmitter
#define ccw_id			0x21		// ID representing counterclockwise ranges in the RF packet
#define cw_id			0x12		// ID representing clockwise ranges in the RF packet

//************************************************************
// Function Declarations
//************************************************************

void initialize(void); 		// Initialization for entire MCU
void uartInit(void);		// Initialization for UART0

//************************************************************
// Variable Declarations
//************************************************************

// RF declarations
unsigned long baudval;			// baud rate for RF control
char rx_data_rf[max_rx_length];	// Received packet
char transmitId;				// Store ID of the RF packet received by the transmitter

// RF decoding declarations
unsigned char vibLed; 			// Leds corresponding to Vibrator Motors
unsigned char rf_id;			// Direction dependent ID for the RF packet 
unsigned char ccw;				// Rotation indication = 0-CounterClockwise, 1-Clockwise
unsigned char rot;  			// Step counter. Represents the step and thus the angle at which the motor is
int range[3];					// Ranges sent by the transmitter of the previously calculated 3 range points

// UART file descriptor
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_WRITE);


//************************************************************
// main() function
//************************************************************

int main(void)
{
  // Initialize entire MCU
  initialize();
  
  // Main task scheduler loop
  while(1)
  {    
	if(rxdone()==1)
	{
		int k = 0;

		// Initializat the RF index variable
		init_getrx();
		
		// Extract all the bytes from the packet received into a data buffer
		while(rx_empty()!=1)
		{
			rx_data_rf[k] = get_next_rx_data();
			k++;
		}

		// Reset the receiver control variables to start the reception of a new RF packet
		rx_reset(max_rx_length); 

		// 0-170, 1-start_char, 2-ID, 3-length, 4-number, 5 to x-payload
	
		// Extract the ID of the RF packet received
		transmitId = decodeOne(rx_data_rf[2]);

		// Check if the RF packet is sent from the Ultrasonic Haptic Radar transmitter board and not some other transmitter
		if(transmitId == rx_id)
		{
			// Extract the different bytes from the RF packet and store them according to their correct semantic representation
			vibLed		 = rx_data_rf[5];
			rf_id 		 = rx_data_rf[6];
			rot 		 = rx_data_rf[7];
			range[0]	 = rx_data_rf[8];
			range[1]	 = rx_data_rf[9];
			range[2]	 = rx_data_rf[10];

			// Turn on the Leds corresponding to the correct vibrator motors
			PORTL = ~vibLed;
		
			// Set direction depending on the rf id received in the RF packet
			if(rf_id == ccw_id)
			{
				ccw = 0;
			}
			else if(rf_id == cw_id)
			{
				ccw = 1;
			}					
	
			// Cases used to calculate the angles for the 3 ranges using the values sent over the RF packet
			if((ccw==0) && (rx_data_rf[7] <= 36) && (rx_data_rf[7] != 0))
			{
				fprintf(stdout,"%d %d %d %d %d %d\n\r", 
				(rx_data_rf[7]+300-4),
				rx_data_rf[8],
				(rx_data_rf[7]+300-2),
				rx_data_rf[9],
				(rx_data_rf[7]+300),
				rx_data_rf[10]);
			}
			else if ((ccw==0) && (rx_data_rf[7] == 0))
			{
				fprintf(stdout,"%d %d %d %d %d %d\n\r", 
				(rx_data_rf[7]+300+4),
				rx_data_rf[8],
				(rx_data_rf[7]+300+2),
				rx_data_rf[9],			
				(rx_data_rf[7]+300),
				rx_data_rf[10]);
			}
			else if((ccw==1) && (rx_data_rf[7] >= 0) && (rx_data_rf[7] != 36))
			{
				fprintf(stdout,"%d %d %d %d %d %d\n\r", 
				(rx_data_rf[7]+300+4),
				rx_data_rf[8],
				(rx_data_rf[7]+300+2),
				rx_data_rf[9],
				(rx_data_rf[7]+300),
				rx_data_rf[10]);
			}
			else if ((ccw==1) && (rx_data_rf[7] == 36))
			{
				fprintf(stdout,"%d %d %d %d %d %d\n\r", 
				(rx_data_rf[7]+300-4),
				rx_data_rf[8],
				(rx_data_rf[7]+300-2),
				rx_data_rf[9],
				(rx_data_rf[7]+300),
				rx_data_rf[10]);
			}

			
		   PORTR ^= (0x04);
	    }		
		
	}

  }
}


//************************************************************
// MCU Initialization Functions
//************************************************************

// Primary Initialization Function
void initialize(void)
{

  // Port Initialization
  // Vibrator motor leds initialization
  DDRL = 0xFF;			// Set all Led connections to outputs
  PORTL = 0x00;			// Initially turn off all Leds

  // RF progress led initialization
  DDRR |= (cpu_l);		// Set the CPU led to output
  PORTR &= ~(cpu_l);	// Initially turn off the CPU led

  
//*********************

  // Variable Initialization
  // RF receiver 
  vibLed = 0;
  rf_id = 0;
  rot = 0;
  ccw = 0;
  transmitId = 0;

//*********************

  // Uart initialization
  uartInit();  

//*********************

  // Start the ISRs
  sei();

}


//**********************************************************

// UART Initialization Function
void uartInit (void)
{

  // Variables for controller the UART initialization
  char tx_u, rx_u, tx_r, rx_r;

  tx_u = 1;	// RS-232 TX
  rx_u = 0; // RS-232 RX

  tx_r = 0; // RF TX
  rx_r = 1; // RF RX
  
  baudval = (f_cpu / (16UL * baudrate)) - 1; 			// Calculate the baudrate for UART0 communication with RS-232 and RF receiver
  txrx_init(0,1,(int)baudval, 1);						// Initialize the control variables in the RF receiver 

  UCSR0B = 0x00;				
  UCSR0B |= ((rx_u<<4)|(tx_u<<3));						// Setting RX/TX enables and interrupts for RS-232
  UCSR0B |= ((rx_r<<7)|(tx_r<<5)|(rx_r<<4)|(tx_r<<3));	// Setting RX/TX enables and interrupts for RF

  UBRR0L = baudval;    									// Setting the baudrate

  stdout = stderr = &uart_str;							// UART variable initialization
  
  // Initial Messages to output when using hyperterm
  fprintf(stdout,"*****  ULTRASONIC HAPTIC VISION *****\n\r"); 
  fprintf(stdout,"***** ECE 4760 Final Project by *****\n\r"); 
  fprintf(stdout,"*****    Rohan Sharma (rs423)   *****\n\r"); 
  fprintf(stdout,"*****    Jeff Buente (jsb89)    *****\n\r"); 
  fprintf(stdout,"RF RECEIVER :  \n\r"); 
  fprintf(stdout,"Starting RF reception...\n\r"); 
}


